# Generated by Django 2.2.12 on 2020-08-15 05:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0028_delete_skilllistfile'),
    ]

    operations = [
        migrations.RenameField(
            model_name='skilllist',
            old_name='Name',
            new_name='name',
        ),
        migrations.AddField(
            model_name='skilllist',
            name='show_on_audit',
            field=models.BooleanField(default=True),
        ),
    ]
