# Generated by Django 3.1.1 on 2020-10-17 07:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0036_auto_20201017_0641'),
    ]

    operations = [
        migrations.CreateModel(
            name='MapSystemPlanet',
            fields=[
                ('planet_id', models.IntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('x', models.FloatField()),
                ('y', models.FloatField()),
                ('z', models.FloatField()),
                ('eve_type', models.ForeignKey(default=None, null=True,
                                               on_delete=django.db.models.deletion.SET_NULL, to='corptools.eveitemtype')),
                ('system', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                             related_name='planet', to='corptools.mapsystem')),
            ],
        ),
        migrations.CreateModel(
            name='MapSystemMoon',
            fields=[
                ('moon_id', models.IntegerField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('x', models.FloatField()),
                ('y', models.FloatField()),
                ('z', models.FloatField()),
                ('system', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                             related_name='moon', to='corptools.mapsystem')),
            ],
        ),
    ]
