# Generated by Django 3.2.5 on 2021-08-08 06:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0014_auto_20210105_1413'),
        ('corptools', '0057_alter_titlefilter_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rolefilter',
            name='alliance_filter',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    related_name='audit_role_filter', to='eveonline.eveallianceinfo'),
        ),
        migrations.AlterField(
            model_name='rolefilter',
            name='corp_filter',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    related_name='audit_role_filter', to='eveonline.evecorporationinfo'),
        ),
    ]
