# Generated by Django 2.2.12 on 2020-08-10 14:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0023_auto_20200810_1350'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mapsystemgate',
            name='from_solar_system',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='from_system', to='corptools.MapSystem'),
        ),
        migrations.AlterField(
            model_name='mapsystemgate',
            name='to_solar_system',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='to_system', to='corptools.MapSystem'),
        ),
    ]
