# Generated by Django 3.2.5 on 2021-07-13 03:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0048_maillabel_mailmessage_mailrecipient'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mailmessage',
            name='is_read',
            field=models.BooleanField(default=False, null=True),
        ),
    ]
